@@ -36,6 +36,7 @@
 #include "chess/board.h"
 #include "chess/callbacks.h"
 #include "chess/position.h"
+#include "neural/cache.h"
 #include "neural/encoder.h"
 #include "neural/writer.h"
 #include "proto/net.pb.h"
@@ -245,8 +246,8 @@ class Node {
       GameResult result, const PositionHistory& history,
       FillEmptyHistory fill_empty_history,
       pblczero::NetworkFormat::InputFormat input_format, Eval best_eval,
-      Eval played_eval, Eval orig_eval, bool best_is_proven, Move best_move,
-      Move played_move) const;
+      Eval played_eval, bool best_is_proven, Move best_move, Move played_move,
+      const NNCacheLock& nneval) const;
 
   // Returns range for iterating over edges.
   ConstIterator Edges() const;
