@@ -27,6 +27,9 @@
 
 // Hack around c++ version incompatibility.
 #include <absl/base/config.h>
+
+#include <cstddef>
+#include <string>
 #undef ABSL_HAVE_STD_STRING_VIEW
 
 #include <tensorflow/cc/client/client_session.h>
@@ -75,29 +78,53 @@ Output MakeIntConst(const Scope& scope, TensorShape shape,
 
 template <bool CPU>
 Output SqueezeAndExcite(const Scope& scope, Input input, int channels,
-                        const LegacyWeights::SEunit& weights) {
+                        const LegacyWeights::SEunit& weights,
+                        const std::string& basename) {
   const int se_channels = weights.b1.size();
   // NCHW ("NHWC" for CPU case) format reduced to NC.
   auto pooled = Mean(scope, input, CPU ? Input({1, 2}) : Input({2, 3}));
   auto w1 = MakeConst(scope, {channels, se_channels}, weights.w1);
   auto b1 = MakeConst(scope, {se_channels}, weights.b1);
-  auto fc1 = Add(scope, MatMul(scope, pooled, w1), b1);
-  auto relu = Relu(scope, fc1);
+  auto dense1_mul = MatMul(scope, pooled, w1);
+  auto dense1_add = Add(scope, dense1_mul, b1);
+  auto relu = Relu(scope, dense1_add);
   auto w2 = MakeConst(scope, {se_channels, 2 * channels}, weights.w2);
   auto b2 = MakeConst(scope, {2 * channels}, weights.b2);
-  auto fc2 = Add(scope, MatMul(scope, relu, w2), b2);
+  auto dense2_mul = MatMul(scope, relu, w2);
+  auto dense2_add = Add(scope, dense2_mul, b2);
   auto reshape = Reshape(
-      scope, fc2,
+      scope, dense2_add,
       CPU ? Input({-1, 1, 1, 2 * channels}) : Input({-1, 2 * channels, 1, 1}));
   auto outputs = Split(scope, CPU ? 3 : 1, reshape, 2);
   auto sigmoid = Sigmoid(scope, outputs[0]);
-  return Add(scope, Mul(scope, sigmoid, input), outputs[1]);
+  auto out_mul = Mul(scope, sigmoid, input);
+  auto out_add = Add(scope, out_mul, outputs[1]);
+
+  pooled.node()->set_name(basename + "/pooled");
+  w1.node()->set_name(basename + "/w1");
+  b1.node()->set_name(basename + "/b1");
+  dense1_mul.node()->set_name(basename + "/dense1_mul");
+  dense1_add.node()->set_name(basename + "/dense1_add");
+  relu.node()->set_name(basename + "/relu");
+  w2.node()->set_name(basename + "/w2");
+  b2.node()->set_name(basename + "/b2");
+  dense2_mul.node()->set_name(basename + "/dense2_mul");
+  dense2_add.node()->set_name(basename + "/dense2_add");
+  reshape.node()->set_name(basename + "/reshape");
+  outputs[0].node()->set_name(basename + "/outputs/1");
+  outputs[0].node()->set_name(basename + "/outputs/2");
+  sigmoid.node()->set_name(basename + "/sigmoid");
+  out_mul.node()->set_name(basename + "/out_mul");
+  out_add.node()->set_name(basename + "/out_add");
+
+  return out_add;
 }
 
 template <bool CPU>
 Output MakeConvBlock(const Scope& scope, Input input, int channels,
                      int input_channels, int output_channels,
                      const LegacyWeights::ConvBlock& weights,
+                     const std::string& basename,
                      const LegacyWeights::SEunit* const seunit = nullptr,
                      Input* mixin = nullptr, bool relu = true) {
   // CPU only supports "NHWC", while for GPU "NCHW" is better.
@@ -105,25 +132,41 @@ Output MakeConvBlock(const Scope& scope, Input input, int channels,
   auto w_conv =
       MakeConst(scope, {channels, channels, input_channels, output_channels},
                 weights.weights, {3, 2, 0, 1});
+  w_conv.node()->set_name(basename + "/weights");
   auto conv2d = Conv2D(scope, input, w_conv, {1, 1, 1, 1}, "SAME",
                        Conv2D::DataFormat(kDataFormat).Dilations({1, 1, 1, 1}));
+  conv2d.node()->set_name(basename + "/conv");
   auto b_conv = MakeConst(scope, {output_channels}, weights.biases);
+  b_conv.node()->set_name(basename + "/weights_biases");
   Output conv_b =
       BiasAdd(scope, conv2d, b_conv, BiasAdd::DataFormat(kDataFormat));
-  if (seunit)
-    conv_b = SqueezeAndExcite<CPU>(scope, conv_b, output_channels, *seunit);
-  if (mixin) conv_b = Add(scope, conv_b, *mixin);
-  return relu ? Relu(scope, conv_b) : conv_b;
+  conv_b.node()->set_name(basename + "/bias_add");
+  if (seunit) {
+    conv_b = SqueezeAndExcite<CPU>(scope, conv_b, output_channels, *seunit,
+                                   basename + "/se");
+  }
+  if (mixin) {
+    conv_b = Add(scope, conv_b, *mixin);
+    conv_b.node()->set_name(basename + "/mixin");
+  }
+  if (relu) {
+    auto out = Relu(scope, conv_b);
+    out.node()->set_name(basename + "/relu");
+    return out;
+  } else {
+    return conv_b;
+  }
 }
 
 template <bool CPU>
 Output MakeResidualBlock(const Scope& scope, Input input, int channels,
-                         const LegacyWeights::Residual& weights) {
-  auto block1 =
-      MakeConvBlock<CPU>(scope, input, 3, channels, channels, weights.conv1);
-  auto block2 =
-      MakeConvBlock<CPU>(scope, block1, 3, channels, channels, weights.conv2,
-                         weights.has_se ? &weights.se : nullptr, &input);
+                         const LegacyWeights::Residual& weights,
+                         const std::string& basename) {
+  auto block1 = MakeConvBlock<CPU>(scope, input, 3, channels, channels,
+                                   weights.conv1, basename + "/conv1");
+  auto block2 = MakeConvBlock<CPU>(
+      scope, block1, 3, channels, channels, weights.conv2, basename + "/conv2",
+      weights.has_se ? &weights.se : nullptr, &input);
   return block2;
 }
 
@@ -134,23 +177,25 @@ std::tuple<Output, Output, Output> MakeNetwork(const Scope& scope, Input input,
   const int filters = weights.input.weights.size() / kInputPlanes / 9;
 
   // Input convolution.
-  auto flow =
-      MakeConvBlock<CPU>(scope, input, 3, kInputPlanes, filters, weights.input);
+  auto flow = MakeConvBlock<CPU>(scope, input, 3, kInputPlanes, filters,
+                                 weights.input, "input/conv");
 
   // Residual tower
-  for (const auto& block : weights.residual) {
-    flow = MakeResidualBlock<CPU>(scope, flow, filters, block);
+  for (size_t i = 0; i < weights.residual.size(); ++i) {
+    const auto& block = weights.residual[i];
+    flow = MakeResidualBlock<CPU>(scope, flow, filters, block,
+                                  "block_" + std::to_string(i));
   }
 
   // Policy head
   Output policy_head;
   if (!weights.policy1.weights.empty()) {
     // Conv policy head.
-    auto conv_pol1 =
-        MakeConvBlock<CPU>(scope, flow, 3, filters, filters, weights.policy1);
+    auto conv_pol1 = MakeConvBlock<CPU>(scope, flow, 3, filters, filters,
+                                        weights.policy1, "policy/conv1");
     auto conv_pol =
         MakeConvBlock<CPU>(scope, conv_pol1, 3, filters, 80, weights.policy,
-                           nullptr, nullptr, /* relu= */ false);
+                           "policy/conv2", nullptr, nullptr, /* relu= */ false);
 
     // [1858 -> HWC or CHW]
     std::vector<int> policy_map(1858);
@@ -171,10 +216,14 @@ std::tuple<Output, Output, Output> MakeNetwork(const Scope& scope, Input input,
     auto flattened_conv =
         Reshape(scope, conv_pol, Const(scope, {-1, 80 * 8 * 8}));
     policy_head = GatherV2(scope, flattened_conv, mapping, 1);
+
+    mapping.node()->set_name("policy/mapping_table");
+    flattened_conv.node()->set_name("policy/flatten");
   } else {
     const int policy_conv_size = weights.policy.biases.size();
-    auto conv_pol = MakeConvBlock<CPU>(scope, flow, 1, filters,
-                                       policy_conv_size, weights.policy);
+    auto conv_pol =
+        MakeConvBlock<CPU>(scope, flow, 1, filters, policy_conv_size,
+                           weights.policy, "policy/conv");
     conv_pol =
         Reshape(scope, conv_pol, Const(scope, {-1, policy_conv_size * 8 * 8}));
     auto ip_pol_w = CPU ? MakeConst(scope, {8, 8, policy_conv_size, 1858},
@@ -188,8 +237,8 @@ std::tuple<Output, Output, Output> MakeNetwork(const Scope& scope, Input input,
   }
 
   // Value head
-  auto conv_val =
-      MakeConvBlock<CPU>(scope, flow, 1, filters, 32, weights.value);
+  auto conv_val = MakeConvBlock<CPU>(scope, flow, 1, filters, 32, weights.value,
+                                     "value/conv");
   conv_val = Reshape(scope, conv_val, Const(scope, {-1, 32 * 8 * 8}));
 
   auto ip1_val_w =
@@ -203,21 +252,28 @@ std::tuple<Output, Output, Output> MakeNetwork(const Scope& scope, Input input,
   if (wdl) {
     auto ip2_val_w = MakeConst(scope, {128, 3}, weights.ip2_val_w);
     auto ip2_val_b = MakeConst(scope, {3}, weights.ip2_val_b);
+    ip2_val_w.node()->set_name("value/w2");
+    ip2_val_b.node()->set_name("value/b2");
     auto ip_fc = Add(scope, MatMul(scope, value_flow, ip2_val_w), ip2_val_b);
     value_head = Softmax(scope, ip_fc);
   } else {
     auto ip2_val_w = MakeConst(scope, {128, 1}, weights.ip2_val_w);
     auto ip2_val_b = MakeConst(scope, {1}, weights.ip2_val_b);
+    ip2_val_w.node()->set_name("value/w2");
+    ip2_val_b.node()->set_name("value/b2");
     auto ip_fc = Add(scope, MatMul(scope, value_flow, ip2_val_w), ip2_val_b);
     value_head = Tanh(scope, ip_fc);
   }
+  ip1_val_w.node()->set_name("value/w1");
+  ip1_val_b.node()->set_name("value/b1");
+  value_flow.node()->set_name("value/relu");
 
   // Moves left head
   Output moves_left_head;
   if (moves_left) {
     const int mlh_channels = weights.moves_left.biases.size();
     auto conv_mov = MakeConvBlock<CPU>(scope, flow, 1, filters, mlh_channels,
-                                       weights.moves_left);
+                                       weights.moves_left, "mlh/conv");
     conv_mov =
         Reshape(scope, conv_mov, Const(scope, {-1, mlh_channels * 8 * 8}));
 
@@ -238,6 +294,9 @@ std::tuple<Output, Output, Output> MakeNetwork(const Scope& scope, Input input,
     moves_left_head = Relu(scope, ip_fc);
   }
 
+  policy_head.node()->set_name("policy/out");
+  value_head.node()->set_name("value/out");
+  moves_left_head.node()->set_name("mlh/out");
   return {policy_head, value_head, moves_left_head};
 }
 
@@ -396,6 +455,7 @@ TFNetwork<CPU>::TFNetwork(const WeightsFile& file, const OptionsDict& options,
         scope_, DataType::DT_FLOAT,
         Placeholder::Shape({-1, kInputPlanes, 8, 8}));
   }
+  input_->node()->set_name("input_planes");
 
   auto output = MakeNetwork<CPU>(scope_, *input_, weights, wdl, IsMlh());
   CHECK(scope_.ok()) << scope_.status().ToString();
@@ -474,8 +534,9 @@ std::unique_ptr<Network> MakeTFNetwork(const std::optional<WeightsFile>& w,
                     " is not supported by Tensorflow C++ backend.");
   }
   return std::make_unique<TFNetwork<CPU>>(
-      weights, options, weights.format().network_format().value() ==
-                            pblczero::NetworkFormat::VALUE_WDL);
+      weights, options,
+      weights.format().network_format().value() ==
+          pblczero::NetworkFormat::VALUE_WDL);
 }
 
 REGISTER_NETWORK("tensorflow-cc-cpu", MakeTFNetwork<true>, 90)
