@@ -25,6 +25,7 @@ class ProtoMessage {
 
   void ParseFromString(std::string_view);
   void MergeFromString(std::string_view);
+  virtual std::string OutputAsString() const = 0;
 
  protected:
   template <class To, class From>
@@ -38,6 +39,15 @@ class ProtoMessage {
     }
   }
 
+  void AppendVarInt(int /* field_id */, std::uint64_t /* value */,
+                    std::string* /* out */) const;
+  void AppendInt64(int /* field_id */, std::uint64_t /* value */,
+                   std::string* /* out */) const;
+  void AppendInt32(int /* field_id */, std::uint32_t /* value */,
+                   std::string* /* out */) const;
+  void AppendString(int /* field_id */, std::string_view /* value */,
+                    std::string* /* out */) const;
+
  private:
   virtual void SetVarInt(int /* field_id */, uint64_t /* value */) {}
   virtual void SetInt64(int /* field_id */, uint64_t /* value */) {}
